# TPK4186 - Advanced Tools for Performance Engineering
# Retailer.py
# by Antoine Rauzy
# Copyrights (c) 2019-2020 NTNU
#
# This file is part of the educational material provided to NTNU students registered to the course TPK4186.
# This work is licensed under a Creative Commons Attribution-NonCommercial 4.0 International License.
# For details about this license, see https://creativecommons.org/licenses/by-nc/4.0/.

# Table of Contents
# -----------------
# 1. Required modules
# 2. Product
# 3. Retailer

# 1. Required modules
# -------------------

import sys
import re

# 2. Product
# ----------

class Product:
  def __init__(self, name):
    self.name = name
    self.profitPerSale = 0
    self.monthlySales = [0 for i in range(0, 12)]

  def GetName(self):
    return self.name

  def GetProfitPerSale(self):
    return self.profitPerSale

  def SetProfitPerSale(self, profitPerSale):
    self.profitPerSale = profitPerSale

  def GetMonthlySales(self, monthNumber):
    if monthNumber<1 or monthNumber>12:
      return 0
    return self.monthlySales[monthNumber-1]

  def SetMonthlySales(self, monthNumber, quantity):
    if monthNumber<1 or monthNumber>12:
      return
    self.monthlySales[monthNumber-1] = quantity

  def ComputeYearlySales(self):
    yearlySales = 0
    for monthNumber in range(1, 13):
      yearlySales += self.GetMonthlySales(monthNumber)
    return yearlySales

  def ComputeMonthlyProfit(self, monthNumber):
    return self.profitPerSale * self.GetMonthlySales(monthNumber)

  def ComputeYearlyProfit(self):
    yearlyProfit = 0.0
    for monthNumber in range(1, 13):
      yearlyProfit += self.ComputeMonthlyProfit(monthNumber)
    return yearlyProfit

# 3. Retailer
# -----------

class Retailer:
  def __init__(self):
    self.products = dict()

  def GetProducts(self):
    return self.products.values()

  def LookForProduct(self, name):
    return self.products.get(name, None)

  def NewProduct(self, name):
    product = self.LookForProduct(name)
    if product==None:
      product = Product(name)
      self.products[name] = product
    return product

  def RecordProduct(self, name, profitPerSale, qtyJan, qtyFeb, qtyMar, qtyApr, qtyMay, qtyJun, qtyJul, qtyAug, qtySep, qtyOct, qtyNov, qtyDec):
    product = self.NewProduct(name)
    product.SetProfitPerSale(profitPerSale)
    product.SetMonthlySales(1, qtyJan)
    product.SetMonthlySales(2, qtyFeb)
    product.SetMonthlySales(3, qtyMar)
    product.SetMonthlySales(4, qtyApr)
    product.SetMonthlySales(5, qtyMay)
    product.SetMonthlySales(6, qtyJun)
    product.SetMonthlySales(7, qtyJul)
    product.SetMonthlySales(8, qtyAug)
    product.SetMonthlySales(9, qtySep)
    product.SetMonthlySales(10, qtyOct)
    product.SetMonthlySales(11, qtyNov)
    product.SetMonthlySales(12, qtyDec)

  def RemoveProduct(self, name):
    self.products.pop(name)

  def ComputeMonthlyProfit(self, monthNumber):
    monthlyProfit = 0.0
    for product in self.products.values():
      monthlyProfit += product.ComputeMonthlyProfit(monthNumber)
    return monthlyProfit

  def ComputeYearlyProfit(self):
    yearlyProfit = 0.0
    for product in self.products.values():
      yearlyProfit += product.ComputeYearlyProfit()
    return yearlyProfit
