# TPK4186 - Advanced Tools for Performance Engineering
# Main.py
# by Antoine Rauzy
# Copyrights (c) 2019-2020 NTNU
#
# This file is part of the educational material provided to NTNU students registered to the course TPK4186.
# It is licensed under a Creative Commons Attribution-NonCommercial 4.0 International License.
# For details about this license, see https://creativecommons.org/licenses/by-nc/4.0/.

# Table of contents
# -----------------
# 1. Required modules
# 2. Loading the cohort
# 3. Queries

# 1. Required modules
# -------------------

from StudentCohort import *

# 2. Loading the cohort
# -----------------------

def CohortCreator():
  cohort = Cohort()
  cohort.RecordStudent(2961201002, "Smith", "Alice", Student.FEMALE, 1, 12, 1996, "alice.smith@nsce.edu", ["MAT4105", "MEC4131", "INF4172", "INF4223", "MAT4260", "MNG4255"])
  cohort.RecordStudent(1970706004, "Johnson", "Bob", Student.MALE, 6, 7, 1997, "bob.johnson@nsce.edu", ["MAT4105", "PYS4160", "INF4172", "INF4223", "PHI4203", "MNG4255"])
  cohort.RecordStudent(2960921005, "Williams", "Carol", Student.FEMALE, 21, 9, 1996, "carol.williams@nsce.edu", ["MAT4105", "PYS4160", "INF4172", "INF4223", "ECO4215", "PHI4203"])
  cohort.RecordStudent(1970217008, "Jones", "David", Student.MALE, 17, 2, 1997, "david.jones@nsce.edu", ["MAT4105", "MEC4131", "INF4172", "INF4223", "PYS4265", "INF4261"])
  cohort.RecordStudent(2950307011, "Brown", "Eve", Student.FEMALE, 7, 3, 1995, "eve.brown@nsce.edu", ["MAT4105", "PYS4160", "INF4172", "INF4223", "MAT4260", "MNG4255"])
  cohort.RecordStudent(1960115013, "Davis", "Frank", Student.MALE, 15, 1, 1996, "frank.davis@nsce.edu", ["MAT4105", "PYS4160", "INF4172", "INF4223", "PYS4265", "PHI4203"])
  cohort.RecordStudent(2970701014, "Miller", "Grace", Student.FEMALE, 1, 7, 1997, "grace.miller@nsce.edu", ["MAT4105", "PYS4160", "INF4172", "INF4223", "PHI4203", "PYS4265"])
  cohort.RecordStudent(2950504017, "Wilson", "Heidi", Student.FEMALE, 4, 5, 1995, "heidi.wilson@nsce.edu", ["MAT4105", "MEC4131", "INF4172", "INF4223", "MAT4240", "PHI4203"])
  cohort.RecordStudent(1970324019, "Hall", "Ivan", Student.MALE, 24, 3, 1997, "ivan.hall@nsce.edu", ["MAT4105", "PYS4160", "INF4172", "INF4223", "PYS4265", "INF4261"])
  cohort.RecordStudent(2970626020, "Allen", "Joan", Student.FEMALE, 26, 6, 1997, "joan.allen@nsce.edu", ["MAT4105", "PYS4160", "INF4172", "INF4223", "MAT4240", "PYS4265"])
  cohort.RecordStudent(1951226023, "Young", "Michael", Student.MALE, 26, 12, 1995, "michael.young@nsce.edu", ["MAT4105", "PYS4160", "INF4172", "INF4223", "ECO4215", "INF4261"])
  cohort.RecordStudent(2960930024, "Hernandez", "Nicole", Student.FEMALE, 30, 9, 1996, "nicole.hernandez@nsce.edu", ["MAT4105", "PYS4160", "INF4172", "INF4223", "MAT4260", "MNG4255"])
  cohort.RecordStudent(1950331025, "King", "Oscar", Student.MALE, 31, 3, 1995, "oscar.king@nsce.edu", ["MAT4105", "MEC4131", "INF4172", "INF4223", "PYS4265", "ECO4215"])
  cohort.RecordStudent(2950711026, "Wright", "Peggy", Student.FEMALE, 11, 7, 1995, "peggy.wright@nsce.edu", ["MAT4105", "PYS4160", "INF4172", "INF4223", "MAT4240", "PHI4203"])
  cohort.RecordStudent(1960213027, "Smith ", "Ruppert", Student.MALE, 13, 2, 1996, "ruppert.smith @nsce.edu", ["MAT4105", "PYS4160", "INF4172", "INF4223", "PHI4203", "INF4261"])
  cohort.RecordStudent(2960422030, "Brown", "Sybil", Student.FEMALE, 22, 4, 1996, "sybil.brown@nsce.edu", ["MAT4105", "PYS4160", "INF4172", "INF4223", "MNG4255", "INF4261"])
  cohort.RecordStudent(1950121032, "Johnson", "Ted", Student.MALE, 21, 1, 1995, "ted.johnson@nsce.edu", ["MAT4105", "MEC4131", "INF4172", "INF4223", "PHI4203", "PYS4265"])
  cohort.RecordStudent(1960907034, "Sanchez", "Victor", Student.MALE, 7, 9, 1996, "victor.sanchez@nsce.edu", ["MAT4105", "PYS4160", "INF4172", "INF4223", "PYS4265", "MAT4260"])
  cohort.RecordStudent(1950806036, "Hernandez", "Walter", Student.MALE, 6, 8, 1995, "walter.hernandez@nsce.edu", ["MAT4105", "PYS4160", "INF4172", "INF4223", "MAT4260", "ECO4215"])
  return cohort

# 3. Queries
# ----------

cohort = CohortCreator()
query = Query()

# Query 0: All students
students = cohort.GetStudents()
print("Q0")
query.PrintStudents(students)

# Query 1: whose family name is Johnson
result = query.Select(students, Student.IsFamilyName, "Johnson")
print("Q1")
query.PrintStudents(result)

# Query 2: Students born in 1995
result = query.Select(students, Student.IsYearOfBirth, 1995)
print("Q2")
query.PrintStudents(result)

# Query 3: Students registered to PYS4160
result = query.Select(students, Student.IsRegistered, "PYS4160")
print("Q3")
query.PrintStudents(result)

# Query 4: Students registered to both PYS4160 and PYS4265
result = query.Select(students, Student.IsRegistered, "PYS4265")
result = query.Select(result, Student.IsRegistered, "PYS4160")
print("Q4")
query.PrintStudents(result)

# Query 5: Students born in March or in April. 
result1 = query.Select(students, Student.IsMonthOfBirth, 3)
result2 = query.Select(students, Student.IsMonthOfBirth, 4)
result = query.Union(result1, result2)
print("Q5")
query.PrintStudents(result)

# Query 6: Students born in 1996 or 1997 who are not taking PYS4265
result1 = query.Select(students, Student.IsYearOfBirth, 1996)
result2 = query.Select(students, Student.IsYearOfBirth, 1997)
result3 = query.Union(result1, result2)
result4 = query.Select(students, Student.IsRegistered, "PYS4265")
result = query.Difference(result3, result4)
print("Q6")
query.PrintStudents(result)

